## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""UI components for students and related components.
"""
import sys
import grok

from datetime import datetime
from zope.formlib.widget import CustomWidgetFactory
from zope.formlib.form import setUpEditWidgets
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.traversing.browser import absoluteURL
from zope.component import (
    createObject,)

from hurry.workflow.interfaces import IWorkflowInfo, IWorkflowState
from reportlab.pdfgen import canvas
from reportlab.lib.units import cm
from reportlab.lib.pagesizes import A4
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import (Frame, Paragraph, Image,
    Table, Spacer)
from reportlab.platypus.tables import TableStyle

from waeup.sirp.accesscodes import invalidate_accesscode, get_access_code
from waeup.sirp.accesscodes.workflow import USED
from waeup.sirp.browser import (
    WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage, WAeUPDisplayFormPage)
from waeup.sirp.browser.breadcrumbs import Breadcrumb
from waeup.sirp.browser.layout import NullValidator
from waeup.sirp.browser.pages import add_local_role, del_local_roles
from waeup.sirp.browser.resources import datepicker, tabs, datatable
from waeup.sirp.browser.viewlets import (
    ManageActionButton, PrimaryNavTab, LeftSidebarLink
    )
from waeup.sirp.image.browser.widget import (
    ThumbnailWidget, EncodingImageFileWidget,
    )
from waeup.sirp.image.image import createWAeUPImageFile
from waeup.sirp.interfaces import IWAeUPObject, ILocalRolesAssignable
from waeup.sirp.permissions import get_users_with_local_roles
from waeup.sirp.university.interfaces import ICertificate
from waeup.sirp.widgets.datewidget import (
    FriendlyDateWidget, FriendlyDateDisplayWidget)
from waeup.sirp.widgets.restwidget import ReSTDisplayWidget
from waeup.sirp.widgets.objectwidget import (
    WAeUPObjectWidget, WAeUPObjectDisplayWidget)
from waeup.sirp.widgets.multilistwidget import (
    MultiListWidget, MultiListDisplayWidget)
from waeup.sirp.students.interfaces import (
    IStudentsContainer, IStudent,
    )
from waeup.sirp.students.students import (
    Student,
    )

class StudentsTab(PrimaryNavTab):
    """Students tab in primary navigation.
    """

    grok.context(IWAeUPObject)
    grok.order(3)
    grok.require('waeup.viewStudents')
    grok.template('primarynavtab')

    pnav = 4
    tab_title = u'Students'

    @property
    def link_target(self):
        return self.view.application_url('students')

class StudentsContainerPage(WAeUPDisplayFormPage):
    """The standard view for regular student containers.
    """
    grok.context(IStudentsContainer)
    grok.name('index')
    grok.require('waeup.viewStudents')
    grok.template('studentscontainerpage')

    form_fields = grok.AutoFields(IStudentsContainer)
    #form_fields['description'].custom_widget = ReSTDisplayWidget

    @property
    def title(self):
        return "Students"

    @property
    def label(self):
        return self.title

class StudentssContainerManageActionButton(ManageActionButton):
    grok.order(1)
    grok.context(IStudentsContainer)
    grok.view(StudentsContainerPage)
    grok.require('waeup.manageStudents')
    text = 'Manage students'

class StudentsContainerManagePage(WAeUPDisplayFormPage):
    """The standard view for regular student containers.
    """
    grok.context(IStudentsContainer)
    grok.name('manage')
    grok.require('waeup.manageStudents')
    grok.template('studentscontainerpage')

    form_fields = grok.AutoFields(IStudentsContainer)

    @property
    def title(self):
        return "Students"

    @property
    def label(self):
        return self.title

    @grok.action('Add student', validator=NullValidator)
    def addStudent(self, **data):
        self.redirect(self.url(self.context, 'addstudent'))
        return

class StudentAddFormPage(WAeUPAddFormPage):
    """Add-form to add a student.
    """
    grok.context(IStudentsContainer)
    grok.require('waeup.manageStudents')
    grok.name('addstudent')
    grok.template('studentaddpage')
    form_fields = grok.AutoFields(IStudent)
    title = 'Students'
    label = 'Add student'

    @grok.action('Create student record')
    def addStudent(self, **data):
        student_id = self.request.form.get('form.student_id')
        student = createObject(u'waeup.Student')
        self.applyData(student, **data)
        #import pdb; pdb.set_trace()
        try:
            self.context[student_id] = student
        except KeyError:
            self.flash('The student id chosen already exists.')
            return
        self.redirect(self.url(self.context[student_id], 'index'))
        return

class DisplayStudent(WAeUPDisplayFormPage):
    grok.context(IStudent)
    grok.name('index')
    grok.require('waeup.viewStudents')
    form_fields = grok.AutoFields(IStudent)

    @property
    def title(self):
        return 'Student: %s' % self.context.name

    @property
    def label(self):
        return self.context.name
